module calculos
!
! Modelo de Transporte de Calor unidimensional em reservatrios
! Implementao: Mtodo de volumes finitos
! Bruna Arcie Polli
! Orientador: Tobias Bleninger
! Mestrado em Engenharia de Recursos Hdricos e Ambiental - PPGERHA, UFPR 2014
! Modulo para o clculo de fluxos de calor na interface, nmero de volumes em cada dt e difuso
!
contains
!
!------------------------------------------------------------------
! delta_z: Subrotina que calcula o nmero de volumes em z
!
subroutine delta_z (j, n, H, m)
!
  use constantes
  implicit none
!
double precision, dimension(n+1):: H
integer:: m					! nmero de volumes
integer:: j					! contador do tempo
integer:: n                 ! nmero de intervalos de tempo 

! Clculo de dz
	m = H(j)/ dz + 2
return
end subroutine delta_z
!------------------------------------------------------------------
! delta_x: Subrotina que calcula o nmero de passos de tempo
!
subroutine delta_t(n)
!
  use constantes 
  implicit none
!
integer:: n				! nmero de intervalos de tempo	

! Clculo de n - nmero de intervalos de tempo
	n = int (tempo / dt)    

return
end subroutine delta_t
!------------------------------------------------------------------------------------------------------------------
! Subrotina para calcular a difuso do calor
subroutine difusao (H, U, m, T, j, n, rho, cp, N2, Ri, Kh)
!
  use constantes
  implicit none
  
! Declarao de variveis
integer:: n
integer:: j
integer:: i									
integer:: m
double precision:: drho					! Diferena de massa especfica de duas camadas
double precision:: ws					! Velocidade de atrito na superfcie do reservatrio
double precision:: r	 				! Equao em funo de U e da latitude
integer:: verifica						! verificao da estratificao trmica
double precision, dimension(m):: T		! Temperatura da gua
double precision, dimension(m):: rho	! Massa especfica da gua
double precision, dimension(m):: cp		! Calor especfico da gua
double precision, dimension (m):: Kh	! coeficiente de difuso turbulenta	
double precision, dimension (m):: z		! Varivel auxiliar
double precision, dimension (m):: N2	! Frequncia de Brunt-Vaisala
double precision, dimension (m):: Ri	! Nmero de Richardson
double precision, dimension(n+1):: H	! Profundidade do reservatrio
double precision, dimension (n+1):: U	! velocidade do vento   
!------------------------------------------------------------------------------------------------------------------------------
do i = 1, m-1
    ! Clculo da massa especfica da gua (Frmula da UNESCO para gua pura)
     rho (i) = (999.842594d0 + 0.06793952d0 * T(i) - 0.009095290d0 * T(i) ** 2 &
                  + 0.0001001685d0 * T(i) ** 3 - 0.000001120083d0 * T(i) ** 4 &
                  + 0.000000006536332d0 * T(i) ** 5) 
   ! Clculo do calor especfico da gua (Frmula da UNESCO)
   cp (i) = (4217.4d0 - 3.720283d0 * T(i) + 0.1412855d0 * T(i) ** 2 -  &
                0.002654387d0 * T(i) ** 3 + 0.00002093236d0 * T(i) ** 4) 
end do
rho (m)=rho (m-1)		! Aproximao
cp (m)= cp (m-1)		! Aproximao
!------------------------------------------------------------------------------------------------------------------------------
! Difuso turbulenta do calor   
verifica=0										! Auxiliar para verificar estratificao trmica
ws = 0.0012d0 * U(j)    						! Velocidade de atrito. Rever essa equao para uma mais precisa    
r = 6.6d0*(sin(phi))**(1./2.)*U(j)**(-1.84)		! funo utilizada em Kh, funo da latitude	
do i = 2, m-2									! verifica a estratificao trmica
  drho = (rho(i+1) - rho(i-1))/(2.d0*dz)
  if (drho < (- 0.1d0/(dz))) then  				! Condio imposta para estratificao => drho > 0.1 kg/m3 (- por causa do sistema de referncia)
     verifica = 1
     exit
  end if
end do  
! Calculo da difuso
do i=1, m-2
  N2(i)=-g / rho(i) * ((rho(i+1)-rho(i))/dz)								! Frequencia de Brunt-Vaisala
  Ri(i) = (kappa**2)*(H(j) - (i - 1) * dz + dz / 2.d0)**2 *N2(i)/ws**2		! Numero de Richardson
  z (i) = kappa * ws * (H(j) - (i - 1) * dz ) / P0							! Auxiliar
  if(verifica==1)then														! est estratificado
    Kh(i)=z(i)*exp(-r*(H(j)-(i-1)*dz))*1.d0/(1.d0+37.*(Ri(i)**2))
    else																	! no est estratificado
      Kh(i)=z(i)*exp(-r*(H(j)-(i-1)*dz))
  end if 
  if(Kh(i)<1.5E-7)then							 ! DIFUSO MOLECULAR; condio para que a difuso no seja menor que a molecular
     Kh(i)=1.5E-7
  end if  
end do 
! Aproximaes 
N2(m)=N2(m-2)
N2(m-1)=N2(m-2)
Ri(m-1) =Ri(m-2)      
Ri(m) =Ri(m-2)
Kh(m) =Kh(m-2)        
Kh(m-1) =Kh(m-2)

return
end subroutine 
!------------------------------------------------------------------------------------------------------------------
! Subrotina para aumentar a difuso do calor com a vazo
subroutine difusao_vazao(m, j, n, H, Qin, Kh_vazao)
!
	use constantes
    implicit none
!
integer:: j
integer:: n
integer:: i
integer:: m
double precision, dimension(n+1):: H					! Profundidade do reservatrio
double precision, dimension (m):: Qin					! vazao em cada volume
double precision, dimension (m):: Kh_vazao				! difuso turbulenta -> vazao

do i=2, m-1
    if(Qin(i)/=0.d0)then
       Kh_vazao(i)=(0.067*H(j)* (0.0233*(Qin(i)/((Bf+2.5*hf)*hf))**2*((8.96E-7)&
       &/(hf*(Qin(i)/((Bf+2.5*hf)*hf))))**0.25)**0.5)*0.01
       else
         Kh_vazao(i)=0.d0
    end if  
end do 

return
end subroutine
!------------------------------------------------------------------------------------------------------------------
! fluxo_qn: Subrotina para determinar o fluxo de calor na superfcie do lago
!
subroutine fluxo_qn (maux, m, n, j, T, U, Ta, UR, qs, qn)				
!
use constantes
implicit none
!
integer:: m
integer:: n
integer:: j
integer, dimension(n+1):: maux
double precision, dimension (m):: T
double precision, dimension (n+1):: U 
double precision, dimension (n+1):: Ta
double precision, dimension (n+1):: UR
double precision, dimension (n+1):: qs
double precision, dimension(n+1):: qn	! Fluxo de energia na superfcie
double precision:: Ts					! Temperatura na superficie do reservatrio
double precision:: fU					! Varivel para o clculo do fluxo de calor na superfcie
double precision:: var					! Varivel para o clculo do fluxo de calor na superfcie
double precision:: Td					! Temperatura do ponto de orvalho
double precision:: Tm					! Temperatura mdia entre Td e Ts
double precision:: delta				! Varivel para o clculo do fluxo de calor na superfcie
double precision:: K 					! Coeficiente de transferncia de calor
double precision:: Te					! Temperatura de equilbrio
double precision:: qns					! radiao de ondas curtas lquida
double precision:: qsn    				! radiao absorvida na superfcie

    qsn=(1-as)*qs(j)
    qns=beta*qsn
    Ts = T(maux(j)-1)
    fU = 9.2d0 + 0.46d0 * U(j) ** 2
    var = (17.27d0 * Ta(j)) / (237.3d0 + Ta(j)) + log (UR(j) / 100.d0)
    Td = (237.3d0 * var) / (17.27d0 - var) 
    Tm = (Ts + Td) / 2.d0
    delta = 0.35d0 + 0.015d0 * Tm + 0.0012d0 * (Tm ** 2)
    K = 4.5d0 + 0.05d0 * Ts + delta * fU + 0.47d0 * fU  
    Te = Td + (qns / K)
    qn(j) = K * (Te - Ts) 

return
end subroutine fluxo_qn
!------------------------------------------------------------------------------------------------------------------
! temperatura_rio : subrotina para calcular a temperatura do rio com base na temperatura do ar
subroutine temperatura_rio (Ta, n, Tas, Tam, Tin)

use constantes

implicit none

double precision, dimension(np):: Tas
double precision, dimension (n+1):: Ta 
double precision, dimension (n+1):: Tam 
double precision, dimension (n+1):: Tin
double precision:: soma
integer:: l
integer:: v
integer:: n
integer:: j
integer:: k
open(unit=22, file='temperatura_inflow.txt')
! Temperatura mdia do ar - para a estimativa da temperatura do rio
! Rotina para calcular a mdia da Ta
    ! Primeiro conjunto de pontos
  	  soma=0.d0
      do j=1, t_saida/dt
           soma=soma+Ta(j)

      end do
      Tas(1)=soma/(t_saida/dt) 
   ! Inicializao de inteiros auxiliares nos clculos
   l=1
   k=2
   do k=2, np
     	 soma=0.d0
         do j=(t_saida/dt)*(l-1)+1, (t_saida/dt)*(l)   
           soma=soma+ Ta(j)   
         end do
         Tas(k)=soma/(t_saida/dt)
         l=l+1   
   end do
   l=1
   do v=1, np
    do j=(86400/dt)*(l-1)+1, (86400/dt)*(l)
           Tam(j)=Tas(v)
    end do
    l=l+1
   end do  
   ! Calculo da temperatura do inflow
   do j=1, n
     Tin(j)=5.0+0.75*Tam(j)
   end do 
   do j=1, n, t_saida/dt
     write(22,*)Tin(j)
   end do  
close(22)
return
end subroutine temperatura_rio
!-----------------------------------------------------------------------------------------
! vento: subrotina para calcular a mdia mvel dos dados de entrada do vento
subroutine vento (n, U0, U)

use constantes
implicit none

integer:: n									! pontos no tempo
double precision, dimension (n+1):: U0		! velocidade do vento sem a mdia mvel
double precision, dimension (n+1):: U		! velocidade do vento COM a mdia mvel
double precision:: soma						! auxiliar no calculo da mdia
integer:: j									! contador do tempo
integer:: k									! contador auxiliar no calculo da mdia  
integer:: x									! auxiliar no calculo da mdia  

x = int(t_media/dt)
! Inicializao de inteiros auxiliares nos clculos
    do j=1, n-x
        soma=0.d0
        do k=j, x+j
            soma=U0(k)+soma 
        end do
        U(j+x/2)=soma/(x+1)
    end do
! Pontos extremos
    do j=1, x/2
      U(j)=U(x/2+1)
    end do
    do j=n-x/2, n
      U(j)=U(n-x/2-1)
    end do 

return
end subroutine vento  
!-----------------------------------------------------------------------------------------
! nivel: subrotina para calcular o nivel para cada dt
subroutine nivel_medio (n, nivel)

use constantes

implicit none

double precision, dimension(np):: ns
integer:: l
integer:: x
integer:: v
integer:: n
integer:: j
double precision:: soma	
double precision, dimension (n+1):: nivel	

open (unit = 22, file = "dados_nivel.txt")
open (unit = 23, file = "nivel_medio.txt")    
do j=1, npm
      read (22,*) nivel(j)			! Entrada com o nvel  em escala diria! Mdia do nvel medido!
end do   	
close(22)
! Determinao do nvel mdio dirio
x = int( t_saida/t_dados_Temp)		! Nmero de dados medidos de nvel dirio
! Inicializao de inteiros auxiliares nos clculos
l=1
do v=1, np
  soma=0.d0
  if(x*l==n)exit
    do j=x*(l-1)+1, x*(l)
      soma=nivel(j)+soma
    end do
    ns(v)=soma/x
    l=l+1
end do    
! Determinao do nivel em cada instante de tempo
l=1
do v=1, np
 do j=(t_saida/dt)*(l-1)+1, (t_saida/dt)*(l)
    	nivel(j)=ns(v)
    end do
    l=l+1
    write(23,*)ns(v)
end do    

return
end subroutine nivel_medio
!-----------------------------------------------------------------------------------------
end module calculos